@@ -330,7 +330,7 @@ class SigV4Auth(BaseSigner):
 class S3SigV4Auth(SigV4Auth):
 
     def _modify_request_before_signing(self, request):
-        super(S3SigV4Auth, self)._add_headers_before_signing(request)
+        super(S3SigV4Auth, self)._modify_request_before_signing(request)
         request.headers['X-Amz-Content-SHA256'] = self.payload(request)
 
     def _normalize_url_path(self, path):
